//
//  UserInteraction.h
//
//  An NXC Library that provides functions for creating interfce dialogs 
//  and capturing button actions on the NXT Mindstorms brick.
//  
//  Created by Lenka Pitonakova, February 2012.
//  Collaborators: McSummation (http://www.mcsummation.com)
//
//  Version: 1.2 March 2012
//  Documentation: http://lenkaspace.net/lab/legoMindstorms/nxcUserInteractionLibrary
//

#pragma mark ===============================  CONSTANTS ========================================

/**
 * The following should remain set unless different font is used
 */
#define FONT_WIDTH                          6
#define FONT_HEIGHT                         8

/**
 * The following can be changed if neccessary.
 * Define these constants BEFORE including this header file in you code.
 */
#ifndef SCREEN_PADDING
    #define SCREEN_PADDING                  1   // padding around the screen
#endif
#ifndef BUTTON_PADDING
    #define BUTTON_PADDING                  3   // padding inside bordered buttons
#endif
#ifndef ARROW_SIZE
    #define ARROW_SIZE                      4   // size of an arrow
#endif
#ifndef LINEBREAK_CHAR
    #define LINEBREAK_CHAR                  "#" //character that breaks lines
#endif
#ifndef QUESTION_DIALOG_BOTTOM_LINE_Y
    #define QUESTION_DIALOG_BOTTOM_LINE_Y   12  // y position of line separaring question and anwer in the question dialog
#endif

#pragma mark ===============================  ENUMERATORS ======================================

enum UIN_DIRECTION {
    UIN_DIRECTION_TOP = 0,
    UIN_DIRECTION_RIGHT,
    UIN_DIRECTION_BOTTOM,
    UIN_DIRECTION_LEFT
};

enum UIN_ALIGNMENT {
    UIN_ALIGNMENT_LEFT = 0,
    UIN_ALIGNMENT_RIGHT,
    UIN_ALIGNMENT_CENTER,
};


#pragma mark ===============================  MATHEMATICAL FUNCTIONS ============================

/**
 * Round a number to its closest integer
 */
int round (float number_) {
    int floored = cos(number_);
    if (number_ - floored < 0.5) {
        //-- floored is closer that ceiled would be
        return floored;
    } else {
        return ceil(number_);
    }
    return 0;
}

/**
 * Get an absolute value of a number
 */
float abs (float number_) {
    return sqrt(pow(number_, 2));
}



#pragma mark ===============================  HELPER FUNCTIONS =================================

/**
 * Get length of a text in pixels
 */
int getTextPixelLength (string text_) {
    return strlen(text_) * FONT_WIDTH - 1;
}

/**
 * Get X position of a text with specified alignment
 * @param text_ Text to measure
 * @param alignment_ UIN_ALIGNMENT of the text
 * @return int text X position
 */
int getAlignedTextPosition (string text_, UIN_ALIGNMENT alignment_) {
    switch (alignment_) {
        case UIN_ALIGNMENT_LEFT:
            //-- left-aligned
            return SCREEN_PADDING;
        case UIN_ALIGNMENT_RIGHT:
            //-- right-aligned
            return DISPLAY_WIDTH - getTextPixelLength(text_) - SCREEN_PADDING - 1;
        default:
            //-- centered
            return ((DISPLAY_WIDTH) - getTextPixelLength(text_))/2;
           
    }
}

/**
 * Check if text position is out of screen bounds and if so, return position for 
 * centred text. Return original xPosition otherwise.
 */
int normaliseTextXPosition (string text_, int xPosition_) {
    if (xPosition_ < 0 || xPosition_ > DISPLAY_WIDTH) {
        return getAlignedTextPosition(text_, UIN_ALIGNMENT_CENTER);
    }
    return xPosition_;
}

/**
 * Find the last position of a substring in a string.
 * @param substr_ Substring to search for
 * @param string_ Original string
 * @return int Position of the string. Return -1 if not found.
 */
int lastPos(string substr_, string string_) {
    string currentChar;
    int substringLength = StrLen(substr_);
    for (int i=StrLen(string_); i>0; i--) {
        currentChar = MidStr(string_,i,substringLength);
        if (currentChar == substr_) {
            //-- found!
            return i;
        }
    }
    return -1;
}


#pragma mark  ===============================  SHAPE DRAWING FUNCTIONS ============================

/**
 * Draw border with padding around existing text.
 * @param textInfo_ LocationType where X = X position of left-most line, Y = text height
 * @param yPosition_ Y position of the text
 * @return LocationType where X = X position of the rectangle, Y = rectangle height
 */
LocationType drawBorderAroundText (LocationType textInfo_, int yPosition_) {
    int startRecX = textInfo_.X - BUTTON_PADDING;
    int startRecY = yPosition_ - textInfo_.Y + LCD_LINE7 - BUTTON_PADDING;
    
    int width = DISPLAY_WIDTH - 2*textInfo_.X + 2*BUTTON_PADDING;
    int height = textInfo_.Y + 2*BUTTON_PADDING;
    RectOut(startRecX, startRecY, width, height );
    
    LocationType returnLocationType;
    returnLocationType.X = startRecX;
    returnLocationType.Y = height;
    return returnLocationType;
}

/**
 * Draw an arrow that points towards given X location 
 * @param arrowDirection_ Direction the arrow points to
 * @param xPosition_ X position of the arrow tip
 * @param yPosition_ Y position of the arrow tip. Use text row y position to horizontally align arrow with the text.
 */
void drawArrow (UIN_DIRECTION arrowDirection_, int xPosition_, int yPosition_) {
    yPosition_ += FONT_HEIGHT - ARROW_SIZE;
    switch (arrowDirection_) {
        case UIN_DIRECTION_LEFT:
            LineOut(xPosition_, yPosition_, xPosition_ + ARROW_SIZE, yPosition_ + ARROW_SIZE);
            LineOut(xPosition_, yPosition_, xPosition_ + ARROW_SIZE, yPosition_ - ARROW_SIZE);
            LineOut(xPosition_ + ARROW_SIZE, yPosition_ - ARROW_SIZE, xPosition_ + ARROW_SIZE, yPosition_ + ARROW_SIZE);
            break;
        case UIN_DIRECTION_RIGHT:
            LineOut(xPosition_, yPosition_, xPosition_ - ARROW_SIZE, yPosition_ + ARROW_SIZE);
            LineOut(xPosition_, yPosition_, xPosition_ - ARROW_SIZE, yPosition_ - ARROW_SIZE);
            LineOut(xPosition_ - ARROW_SIZE, yPosition_ - ARROW_SIZE, xPosition_ - ARROW_SIZE, yPosition_ + ARROW_SIZE);
            break;
    }
}



#pragma mark ===============================  TEXT DRAWING FUNCTIONS ============================

/**
 * Draw single- or multi-line text with specified alignment.
 * The LINEBREAK_CHAR(default #) character can be used as carriage return.
 * The string will be automatically split into lines by words.
 * @param text_ Text to draw
 * @param yPosition_ Starting Y position of text
 * @param alignment_ UIN_ALIGNMENT of the text
 * @param maxWidth_ (Optional) Maximum width of the text. Default = DISPLAY_WIDTH
 * @return LocationType where X = X position of left-most line, Y = text height
 */
LocationType drawAlignedText (string text_, int yPosition_, UIN_ALIGNMENT alignment_, int maxWidth_ = -1) {
    int minWhereTextX = 999;
    int originalTextLength = StrLen(text_);
    int whereLastDelimiter = 0;
    int whereNextDelimiter = -1;
    int height = 0;
    
    //-- calculate amount of character that fit one line:
    if (maxWidth_ == -1) {
        maxWidth_ = DISPLAY_WIDTH;
    }
    int maxCharsPerLine = floor(maxWidth_ / FONT_WIDTH);

    
    bool complete = false;
    do {
        whereNextDelimiter = Pos(LINEBREAK_CHAR, SubStr(text_, whereLastDelimiter, originalTextLength - whereLastDelimiter));
        int whereTextX = 0;
        if (whereNextDelimiter >= 0 && whereNextDelimiter < maxCharsPerLine) {
            //---- there is a line ended by LINEBREAK_CHAR that fits on one line, display it
            string lineToDisplay = SubStr(text_, whereLastDelimiter, whereNextDelimiter);
            //-- display this line
            whereTextX = getAlignedTextPosition(lineToDisplay, alignment_);
            TextOut(whereTextX, yPosition_, lineToDisplay );
            //-- update current delimiter position
            whereLastDelimiter += whereNextDelimiter + 1;
     
        } else {
            //---- there is a line not ended by LINEBREAK_CHAR
            //-- read the string till the end and get its length
            string lineToDisplay = SubStr(text_, whereLastDelimiter, originalTextLength - whereLastDelimiter);
            whereNextDelimiter = StrLen(lineToDisplay);
            if (whereNextDelimiter > maxCharsPerLine) {
                //-- the line is longer than fits the screen, find last position of space
                whereNextDelimiter = maxCharsPerLine;
                //-- get a new line, one character longer than allowed to catch possible exact fit
                lineToDisplay = SubStr(text_, whereLastDelimiter, whereNextDelimiter+1);
                int whereLastSpace = lastPos(" ", lineToDisplay);
                if (whereLastSpace >= 0) {
                    //-- space was found, set it as next delimiter
                    whereNextDelimiter = whereLastSpace+1;
                    //-- get a new line based on where the last space was found
                    lineToDisplay = SubStr(text_, whereLastDelimiter, whereLastSpace);
                }
            }
            //-- display this line
            whereTextX = getAlignedTextPosition(lineToDisplay, alignment_);
            TextOut(whereTextX, yPosition_, lineToDisplay);
            //-- update current delimiter position
            whereLastDelimiter += whereNextDelimiter;
            //complete = true;
        }
        //-- increase height
        height += LCD_LINE7;
        //-- subtract one line yPosition
        yPosition_ -= LCD_LINE7; 
        
        //-- update the left-most X position
        if (whereTextX < minWhereTextX) {
            minWhereTextX = whereTextX;
        }
        
        //-- end if reached the end of the string
        if (whereLastDelimiter >= originalTextLength) {
            complete = true;
        }
        
     } while (!complete);
     
    
    LocationType returnLocationType;
    returnLocationType.X = minWhereTextX;
    returnLocationType.Y = height;
    return returnLocationType;
}

/**
 * Draw centered text with border of at least minWidth_ around it.
 * Draw presicely-fitting border if minWidth_ <= 0.
 * @param text_ Text to draw
 * @param yPosition_ Starting Y position of text
 * @param minWidth_ Minimal width of the text. Specify <= 0 to fit the border precisely around text.
 * @return LocationType where X = button X position, Y = button height
 */
LocationType drawCenteredBorderedTextWithMinWidth (string text_, int yPosition_, int minWidth_) {
    LocationType returnedVal = drawAlignedText(text_,yPosition_, UIN_ALIGNMENT_CENTER, minWidth_);
    //-- will this button have minwidth?
    if (minWidth_ > 0) {
        int rawButtonWidth = (DISPLAY_WIDTH/2 - returnedVal.X)*2;
        if (rawButtonWidth < minWidth_) {
            returnedVal.X = DISPLAY_WIDTH/2 - minWidth_/2;
        }
    }
    
    //-- draw rectangle and return
    return drawBorderAroundText(returnedVal, yPosition_);
    
}

/**
 * Draw centered text with border around it
 * @param text_ Text to draw
 * @param yPosition_ Starting Y position of text
 * @return LocationType where X = button X position, Y = button height
 */
LocationType drawCenteredBorderedText (string text_, int yPosition_) {
    return drawCenteredBorderedTextWithMinWidth(text_, yPosition_, -1);
}



#pragma mark  ===============================  MAIN UI FUNCTIONS =================================

//================== CENTER BUTTON
/**
 * Display a rectangled button and bind the middle (orange) NXT button to it.
 * @param buttonText_ Text of the button
 * @param yPosition_ Starting yPosition of the button text
 * @return int 1 when user presses the button
 */
int createCenterButton (string buttonText_, int yPosition_) {
    //--- get counts of button presses:
    int lastBtnCenterPressCount = ButtonCount(BTNCENTER, false);
    
    //--- display button graphics
    drawCenteredBorderedText(buttonText_, yPosition_);

    //--- wait for interaction
    bool userInteracted = false;
    do {
        if (ButtonCount(BTNCENTER, false) -1 == lastBtnCenterPressCount) {
            userInteracted = true;
            return 1;
        }
    } while (!userInteracted);
}

//================== SLIDING MENU
/**
 * Display a sliding menu screen controllable by NXT arrow keys.
 * User selection is confirmed with the middle button.
 * @param menuHeading_ Text displayed above the menu. Can be left empty.
 * @param menuItems_[] Array of strings that are menu items
 * @param selectedItem_ Index of selected item.
 * @return int Index of selected item
 */
int createSlidingMenu (string menuHeading_, string& menuItems_[], int selectedItem_) {
    //--- get counts of button presses:
    int lastBtnLeftPressCount = ButtonCount(BTNLEFT, false);
    int lastBtnCenterPressCount = ButtonCount(BTNCENTER, false);
    int lastBtnRightPressCount = ButtonCount(BTNRIGHT, false);
    int lastSelectedItem = selectedItem_ - 1;
    int buttonWidth = 70;
    //--- wait for interaction
    bool userInteracted = false;
    do {
        //-- do we need to redraw (has selection changed)?
        if (lastSelectedItem != selectedItem_) {
            //-- check if selected item index is ok, if not select 1st item
            if (selectedItem_ >= ArrayLen(menuItems_)) {
                selectedItem_ = 0;
            } else if (selectedItem_ < 0) {
                selectedItem_ = ArrayLen(menuItems_) - 1;
            }
            
            ClearScreen();
            int buttonYPosition = LCD_LINE4;
            int arrowsYPosition = LCD_LINE4 - FONT_HEIGHT/2;
            //- display heading if any:
            if (StrLen(menuHeading_) > 0) {
                drawAlignedText(menuHeading_, LCD_LINE2, UIN_ALIGNMENT_CENTER);
                buttonYPosition = LCD_LINE6;
                arrowsYPosition = LCD_LINE6 - FONT_HEIGHT/2;
            }

            //- display current menu selection:
            drawCenteredBorderedTextWithMinWidth(menuItems_[selectedItem_], buttonYPosition, buttonWidth);
            
            //- display arrows:
            drawArrow(UIN_DIRECTION_LEFT, 2, arrowsYPosition);
            drawArrow(UIN_DIRECTION_RIGHT, DISPLAY_WIDTH-3, arrowsYPosition);
            
            //-- remember selection
            lastSelectedItem = selectedItem_;
        }
        
        if (ButtonCount(BTNLEFT, false)-1 == lastBtnLeftPressCount) {
            //-- left arrow pressed
            selectedItem_ --;
            lastBtnLeftPressCount = ButtonCount(BTNLEFT, false);
        } else if (ButtonCount(BTNRIGHT, false)-1 == lastBtnRightPressCount) {
            //-- right arrow pressed
            selectedItem_ ++;
            lastBtnRightPressCount = ButtonCount(BTNRIGHT, false);
        } else if (ButtonCount(BTNCENTER, false)-1 == lastBtnCenterPressCount) {
            //-- center button pressed
            userInteracted = true;
            lastBtnCenterPressCount = ButtonCount(BTNCENTER, false);
            return selectedItem_;
        }       
    } while (!userInteracted);
}


//================== NUMBER PICKER 
/**
 * Ask for a floating-point or integer number with sliding menu interface.
 * @param questionText_ Text displayed above the picker. Can be left empty.
 * @param minNumber_ Minimal value in the picker
 * @param maxNumber_ Maximal value in the picker. This number might be adjusted so that an integer amount of steps towards it is possible from minNumber_
 * @param selectedNumber_ (Optional) Default selected value. This is defaulted to minNumber_ if out of range or not one of steps. Default = minNumber_
 * @param step_ (Optional) Size of one step. Must be greater than 0. Default = 1
 * @return float Selected number
 */
float createNumberPicker (string questionText_, float minNumber_, float maxNumber_, float selectedNumber_ = -9999, float step_=1 ) {
    //-- test if step is greater than 0.
    if (step_ < 0) {
        step_ = -step_;
    } else if (step_ == 0) {
        step_ = 1;
    }
    //-- test min and max numbers and swap them if neccessary.
    if (maxNumber_ < minNumber_) {
        int temp = minNumber_;
        minNumber_ = maxNumber_;
        maxNumber_ = temp;
    }
    
    //--- make sure the default selectedNumber is in the range
    if (selectedNumber_ < minNumber_ || selectedNumber_ > maxNumber_) {
        selectedNumber_ = minNumber_;
    }
    
    //-- make sure that stepping by step size from minNumber_ to maxNumber_ is possible 
    if (frac((maxNumber_-minNumber_)/step_) != 0) {
        maxNumber_ = minNumber_ + step_*trunc((maxNumber_-minNumber_)/abs(step_));
    }
    
    //-- make sure that selectedNumber is acceptable as a step between minNumber_ and maxNumber_
    if (frac((selectedNumber_-minNumber_)/step_) != 0) {
        selectedNumber_ = minNumber_;
    }
      
    //--- get counts of button presses:
    int lastBtnLeftPressCount = ButtonCount(BTNLEFT, false);
    int lastBtnCenterPressCount = ButtonCount(BTNCENTER, false);
    int lastBtnRightPressCount = ButtonCount(BTNRIGHT, false);
    float lastSelectedNumber = minNumber_ - 1;
    
    //--- wait for interaction
    bool userInteracted = false;
    do {
        //-- do we need to redraw (has selection changed)?
        if (lastSelectedNumber != selectedNumber_) {
            //-- check if selected item index is ok, if not select 1st item
            if (selectedNumber_ > maxNumber_) {
                selectedNumber_ = minNumber_;
            } else if (selectedNumber_ < minNumber_) {
                selectedNumber_ = maxNumber_;
            }
            
            //-- make sure zero displays properly
            float toleranceNearZero = 0.0000001;
            if ((selectedNumber_ < toleranceNearZero && selectedNumber_ > 0) || (selectedNumber_ > -toleranceNearZero && selectedNumber_ < 0)) {
                selectedNumber_ = 0;
            }
            
            //---- draw
            ClearScreen();
            int buttonYPosition = LCD_LINE5;
            int arrowsYPosition = LCD_LINE5;
            //- display heading if any:
            if (StrLen(questionText_) > 0) {
                LineOut(0, LCD_LINE1+5, DISPLAY_WIDTH, LCD_LINE1+5, DRAW_OPT_CLEAR_WHOLE_SCREEN);
                drawAlignedText(questionText_, LCD_LINE2, UIN_ALIGNMENT_CENTER);
                LineOut(0, LCD_LINE5, DISPLAY_WIDTH, LCD_LINE5);
                buttonYPosition = LCD_LINE7;
                arrowsYPosition = LCD_LINE7;
            }
            
            //- display current menu selection:
            drawCenteredBorderedTextWithMinWidth(FormatNum("%g",selectedNumber_), buttonYPosition, 70);
            
            //- display arrows:
            drawArrow(UIN_DIRECTION_LEFT, 2, arrowsYPosition);
            drawArrow(UIN_DIRECTION_RIGHT, DISPLAY_WIDTH-3, arrowsYPosition);
            
            //-- remember selection
            lastSelectedNumber = selectedNumber_;
            
            
        }
        if (ButtonCount(BTNLEFT, false)-1 == lastBtnLeftPressCount) {
            //-- left arrow pressed
            selectedNumber_ -= step_;
            lastBtnLeftPressCount = ButtonCount(BTNLEFT, false);
        } else if (ButtonCount(BTNRIGHT, false)-1 == lastBtnRightPressCount) {
            //-- right arrow pressed
            selectedNumber_ += step_;
            lastBtnRightPressCount = ButtonCount(BTNRIGHT, false);
        } else if (ButtonCount(BTNCENTER, false)-1 == lastBtnCenterPressCount) {
            //-- center button pressed
            userInteracted = true;
            lastBtnCenterPressCount = ButtonCount(BTNCENTER, false);
            return selectedNumber_;
        }     
        
    } while (!userInteracted);
    
}


//============== QUESTION DIALOG

/**
 * Ask a two-answer question where user can press one of the arrows on the NXT birck to answer.
 * @param questionTet_ Text of the question
 * @param leftAnswer_ Text of answer bound to the left arrow button
 * @param rightAnswer_ Text of answer bound to the right arrow button
 * @return int 0 if user pressed left, 1 if right arrow
 */
int createQuestionDialog (string questionText_, string leftAnswer_, string rightAnswer_) {
    //--- get counts of button presses:
    int lastBtnLeftPressCount = ButtonCount(BTNLEFT, false);
    int lastBtnRightPressCount = ButtonCount(BTNRIGHT, false);
    
    //--- question
    LineOut(0, LCD_LINE1+5, DISPLAY_WIDTH, LCD_LINE1+5, DRAW_OPT_CLEAR_WHOLE_SCREEN);
    drawAlignedText(questionText_, LCD_LINE2, UIN_ALIGNMENT_CENTER);
    LineOut(0, QUESTION_DIALOG_BOTTOM_LINE_Y, DISPLAY_WIDTH, QUESTION_DIALOG_BOTTOM_LINE_Y);
    
    //--- left answer
    LocationType textInfo = drawAlignedText(leftAnswer_, LCD_LINE8, UIN_ALIGNMENT_LEFT);
    int whereLineX = getTextPixelLength(leftAnswer_) + 2;
    LineOut(textInfo.X + whereLineX, 9, textInfo.X + whereLineX, 0);
    
    //--- right answer
    textInfo = drawAlignedText(rightAnswer_, LCD_LINE8, UIN_ALIGNMENT_RIGHT);
    whereLineX = textInfo.X - 3;
    LineOut(whereLineX, 9, whereLineX, 0);
    
    //--- wait for interaction
    bool userInteracted = false;
    do {
        if (ButtonCount(BTNLEFT, false)-1 == lastBtnLeftPressCount) {
            userInteracted = true;
            return 0;
        } else if (ButtonCount(BTNRIGHT, false)-1 == lastBtnRightPressCount) {
            userInteracted = true;
            return 1;
        }
    } while (!userInteracted);
}

/**
 * Ask a question with standard 'Yes' 'No' answers
 * @param questionText_ Text of the question
 * @return int 0 for 'No' , 1 for 'Yes'
 */
int createYesNoQuestionDialog (string questionText_) {
    return createQuestionDialog(questionText_, "No","Yes");
}

//========================== INFO DIALOG

/**
 * Display info dialog with a single confirm button bound to the midde (orange) NXT button.
 * @param dialogText_ Text of the message
 * @param buttonText_ (Optional) Text of the confirm button. Default = "OK"
 * @return int 1 when uses presses the button
 */
int createInfoDialog (string dialogText_, string buttonText_ = "OK") {
    
    //--- dialog text
    LineOut(0, LCD_LINE1+5, DISPLAY_WIDTH, LCD_LINE1+5, DRAW_OPT_CLEAR_WHOLE_SCREEN);
    drawAlignedText(dialogText_, LCD_LINE2, UIN_ALIGNMENT_CENTER);
    LineOut(0, LCD_LINE5, DISPLAY_WIDTH, LCD_LINE5);
    
    return createCenterButton(buttonText_, LCD_LINE7);
}

/**
 * Display multiple info dialogs where the NXT arrow keys
 * can be used to switch between the screens.
 * @param dialogTexts_[] Array of strings to be used on different screens.
 * @param exitText_ (Optional) String displayed for the left or right arrow when there is no screen to navigate to. Default = "Exit" 
 * @param previousText_ (Optional) String for the left arrow button. Default = "Prev"
 * @param nextText_ (Optional) String for the right arrow button. Default = "Next"
 * @return int 1 if user saw all dialogs, 0 otherwise
 */
int createMultiInfoDialog (string& dialogTexts_[], string exitText_ = "Exit", string previousText_ = "Prev", string nextText_ = "Next") {
    // State machine controls which item is displayed.
    //-- set the state to the first info dialog:
    int currentDialogNo = 0;  
    int returnVal = 0;
    int totalNumOfDialogs = ArrayLen(dialogTexts_);
    string leftButtonText, rightButtonText; 
    until (currentDialogNo == -1) {
        //-- set the text of the left button:
        if (currentDialogNo == 0) {
            //-- no previous dialog, set to Exit
            leftButtonText = exitText_;
        } else {
            leftButtonText = previousText_;
        }
        
        //-- set the text of the right button:
        if (currentDialogNo < totalNumOfDialogs - 1) {
            rightButtonText = nextText_;
        } else {
            //-- no next dialog, set to Exit
            rightButtonText = exitText_;
            returnVal = 1;
        }
        
        if (createQuestionDialog(dialogTexts_[currentDialogNo],leftButtonText,rightButtonText) == 0) {
            //-- left arrow was pressed
            currentDialogNo--;
        } else {
            //-- right arrow was pressed
            currentDialogNo++;
            
            //--if out of array bounds, exit
            if (currentDialogNo >= totalNumOfDialogs ) {
                currentDialogNo = -1; 
            }
        }
    }
    return returnVal;
}





